Class: Gem::ConfigFile
- Inherits:
-
Object
- Object
- Gem::ConfigFile
- Includes:
- UserInteraction
- Defined in:
- lib/rubygems/config_file.rb
Overview
Gem::ConfigFile RubyGems options and gem command options from gemrc.
gemrc is a YAML file that uses strings to match gem command arguments and symbols to match RubyGems options.
Gem command arguments use a String key that matches the command name and allow you to specify default arguments:
install: --no-rdoc --no-ri
update: --no-rdoc --no-ri
You can use gem: to set default arguments for all commands.
RubyGems options use symbol keys. Valid options are:
:backtrace:: See #backtrace
:bulk_threshold:: See #bulk_threshold
:verbose:: See #verbose
:update_sources:: See #update_sources
:concurrent_downloads:: See #concurrent_downloads
:cert_expiration_length_days:: See #cert_expiration_length_days
:install_extension_in_lib:: See #install_extension_in_lib
:ipv4_fallback_enabled:: See #ipv4_fallback_enabled
:gemhome:: See #home
:gempath:: See #path
:sources:: Sets Gem::sources
:disable_default_gem_server:: See #disable_default_gem_server
:ssl_verify_mode:: See #ssl_verify_mode
:ssl_ca_cert:: See #ssl_ca_cert
:ssl_client_cert:: See #ssl_client_cert
gemrc files may exist in various locations and are read and merged in the following order:
- system wide (/etc/gemrc)
- per user (~/.gemrc)
- per environment (gemrc files listed in the GEMRC environment variable)
Constant Summary collapse
- DEFAULT_BACKTRACE =
true- DEFAULT_BULK_THRESHOLD =
1000- DEFAULT_VERBOSITY =
true- DEFAULT_UPDATE_SOURCES =
true- DEFAULT_CONCURRENT_DOWNLOADS =
8- DEFAULT_CERT_EXPIRATION_LENGTH_DAYS =
365- DEFAULT_IPV4_FALLBACK_ENABLED =
false- DEFAULT_INSTALL_EXTENSION_IN_LIB =
TODO: Use false as default value for this option in RubyGems 4.0
true- OPERATING_SYSTEM_DEFAULTS =
For Ruby packagers to set configuration defaults. Set in rubygems/defaults/operating_system.rb
Gem.
- PLATFORM_DEFAULTS =
For Ruby implementers to set configuration defaults. Set in rubygems/defaults/#RUBY_ENGINE.rb
Gem.platform_defaults
- SYSTEM_CONFIG_PATH =
:stopdoc:
begin require "etc" Etc.sysconfdir rescue LoadError, NoMethodError RbConfig::CONFIG["sysconfdir"] || "/etc" end
- SYSTEM_WIDE_CONFIG_FILE =
:startdoc:
File.join SYSTEM_CONFIG_PATH, "gemrc"
Instance Attribute Summary collapse
-
#args ⇒ Object
readonly
List of arguments supplied to the config file object.
-
#backtrace ⇒ Object
True if the backtrace option has been specified, or debug is on.
-
#bulk_threshold ⇒ Object
Bulk threshold value.
-
#cert_expiration_length_days ⇒ Object
Expiration length to sign a certificate.
-
#concurrent_downloads ⇒ Object
Number of gem downloads that should be performed concurrently.
-
#disable_default_gem_server ⇒ Object
True if we want to force specification of gem server when pushing a gem.
-
#home ⇒ Object
Where to install gems (deprecated).
-
#install_extension_in_lib ⇒ Object
Install extensions into lib as well as into the extension directory.
-
#ipv4_fallback_enabled ⇒ Object
Experimental == Fallback to IPv4 when IPv6 is not reachable or slow (default: false).
-
#path ⇒ Object
Where to look for gems (deprecated).
-
#sources ⇒ Object
sources to look for gems.
-
#ssl_ca_cert ⇒ Object
Path name of directory or file of openssl CA certificate, used for remote https connection.
-
#ssl_client_cert ⇒ Object
readonly
Path name of directory or file of openssl client certificate, used for remote https connection with client authentication.
-
#ssl_verify_mode ⇒ Object
readonly
openssl verify mode value, used for remote https connection.
-
#update_sources ⇒ Object
True if we want to update the SourceInfoCache every time, false otherwise.
-
#verbose ⇒ Object
Verbose level of output: * false -- No output * true -- Normal output * :loud -- Extra output.
Class Method Summary collapse
- .deep_transform_config_keys!(config) ⇒ Object
- .dump_with_rubygems_yaml(content) ⇒ Object
- .load_with_rubygems_config_hash(yaml) ⇒ Object
Instance Method Summary collapse
-
#==(other) ⇒ Object
:nodoc:.
-
#[](key) ⇒ Object
Return the configuration information for
key. -
#[]=(key, value) ⇒ Object
Set configuration option
keytovalue. -
#api_keys ⇒ Object
Hash of RubyGems.org and alternate API keys.
-
#check_credentials_permissions ⇒ Object
Checks the permissions of the credentials file.
-
#config_file_name ⇒ Object
The name of the configuration file.
-
#credentials_path ⇒ Object
Location of RubyGems.org credentials.
-
#each {|:update_sources, @update_sources| ... } ⇒ Object
Delegates to @hash.
-
#handle_arguments(arg_list) ⇒ Object
Handle the command arguments.
-
#initialize(args) ⇒ ConfigFile
constructor
Create the config file object.
-
#last_update_check ⇒ Object
Reads time of last update check from state file.
-
#last_update_check=(timestamp) ⇒ Object
Writes time of last update check to state file.
- #load_api_keys ⇒ Object
- #load_file(filename) ⇒ Object
-
#really_verbose ⇒ Object
Really verbose mode gives you extra output.
-
#rubygems_api_key ⇒ Object
Returns the RubyGems.org API key.
-
#rubygems_api_key=(api_key) ⇒ Object
Sets the RubyGems.org API key to
api_key. -
#set_api_key(host, api_key) ⇒ Object
Set a specific host's API key to
api_key. -
#state_file_name ⇒ Object
The name of the state file.
-
#state_file_writable? ⇒ Boolean
Check state file is writable.
-
#to_yaml ⇒ Object
to_yaml only overwrites things you can't override on the command line.
-
#unset_api_key! ⇒ Object
Remove the
~/.gem/credentialsfile to clear all the current sessions. -
#write ⇒ Object
Writes out this config file, replacing its source.
Methods included from UserInteraction
#alert, #alert_error, #alert_warning, #ask, #ask_for_password, #ask_yes_no, #choose_from_list, #say, #terminate_interaction
Methods included from DefaultUserInteraction
ui, #ui, ui=, #ui=, use_ui, #use_ui
Methods included from Text
#clean_text, #format_text, #levenshtein_distance, #min3, #truncate_text
Constructor Details
#initialize(args) ⇒ ConfigFile
Create the config file object. args is the list of arguments
from the command line.
The following command line options are handled early here rather than later at the time most command options are processed.
- --config-file, --config-file==NAME
Obviously these need to be handled by the ConfigFile object to ensure we get the right config file.
- --backtrace
Backtrace needs to be turned on early so that errors before normal option parsing can be properly handled.
- --debug
Enable Ruby level debug messages. Handled early for the same reason as --backtrace.
-- TODO: parse options upstream, pass in options directly
195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 |
# File 'lib/rubygems/config_file.rb', line 195 def initialize(args) set_config_file_name(args) @backtrace = DEFAULT_BACKTRACE @bulk_threshold = DEFAULT_BULK_THRESHOLD @verbose = DEFAULT_VERBOSITY @update_sources = DEFAULT_UPDATE_SOURCES @concurrent_downloads = DEFAULT_CONCURRENT_DOWNLOADS @cert_expiration_length_days = DEFAULT_CERT_EXPIRATION_LENGTH_DAYS @install_extension_in_lib = DEFAULT_INSTALL_EXTENSION_IN_LIB @ipv4_fallback_enabled = ENV["IPV4_FALLBACK_ENABLED"] == "true" || DEFAULT_IPV4_FALLBACK_ENABLED = Gem::Util.deep_dup(OPERATING_SYSTEM_DEFAULTS) platform_config = Gem::Util.deep_dup(PLATFORM_DEFAULTS) system_config = load_file SYSTEM_WIDE_CONFIG_FILE user_config = load_file config_file_name environment_config = (ENV["GEMRC"] || ""). split(File::PATH_SEPARATOR).inject({}) do |result, file| result.merge load_file file end @hash = .merge platform_config unless args.index "--norc" @hash = @hash.merge system_config @hash = @hash.merge user_config @hash = @hash.merge environment_config end @hash.transform_keys! do |k| # gemhome and gempath are not working with symbol keys if %w[backtrace bulk_threshold verbose update_sources cert_expiration_length_days install_extension_in_lib ipv4_fallback_enabled sources disable_default_gem_server ssl_verify_mode ssl_ca_cert ssl_client_cert].include?(k) k.to_sym else k end end # HACK: these override command-line args, which is bad @backtrace = @hash[:backtrace] if @hash.key? :backtrace @bulk_threshold = @hash[:bulk_threshold] if @hash.key? :bulk_threshold @verbose = @hash[:verbose] if @hash.key? :verbose @update_sources = @hash[:update_sources] if @hash.key? :update_sources # TODO: We should handle concurrent_downloads same as other options @cert_expiration_length_days = @hash[:cert_expiration_length_days] if @hash.key? :cert_expiration_length_days @install_extension_in_lib = @hash[:install_extension_in_lib] if @hash.key? :install_extension_in_lib @ipv4_fallback_enabled = @hash[:ipv4_fallback_enabled] if @hash.key? :ipv4_fallback_enabled @home = @hash[:gemhome] if @hash.key? :gemhome @path = @hash[:gempath] if @hash.key? :gempath @sources = @hash[:sources] if @hash.key? :sources @disable_default_gem_server = @hash[:disable_default_gem_server] if @hash.key? :disable_default_gem_server @ssl_verify_mode = @hash[:ssl_verify_mode] if @hash.key? :ssl_verify_mode @ssl_ca_cert = @hash[:ssl_ca_cert] if @hash.key? :ssl_ca_cert @ssl_client_cert = @hash[:ssl_client_cert] if @hash.key? :ssl_client_cert @api_keys = nil @rubygems_api_key = nil handle_arguments args end |
Instance Attribute Details
#args ⇒ Object (readonly)
List of arguments supplied to the config file object.
93 94 95 |
# File 'lib/rubygems/config_file.rb', line 93 def args @args end |
#backtrace ⇒ Object
True if the backtrace option has been specified, or debug is on.
399 400 401 |
# File 'lib/rubygems/config_file.rb', line 399 def backtrace @backtrace || $DEBUG end |
#bulk_threshold ⇒ Object
Bulk threshold value. If the number of missing gems are above this threshold value, then a bulk download technique is used. (deprecated)
114 115 116 |
# File 'lib/rubygems/config_file.rb', line 114 def bulk_threshold @bulk_threshold end |
#cert_expiration_length_days ⇒ Object
Expiration length to sign a certificate
156 157 158 |
# File 'lib/rubygems/config_file.rb', line 156 def cert_expiration_length_days @cert_expiration_length_days end |
#concurrent_downloads ⇒ Object
Number of gem downloads that should be performed concurrently.
127 128 129 |
# File 'lib/rubygems/config_file.rb', line 127 def concurrent_downloads @concurrent_downloads end |
#disable_default_gem_server ⇒ Object
True if we want to force specification of gem server when pushing a gem
137 138 139 |
# File 'lib/rubygems/config_file.rb', line 137 def disable_default_gem_server @disable_default_gem_server end |
#home ⇒ Object
Where to install gems (deprecated)
103 104 105 |
# File 'lib/rubygems/config_file.rb', line 103 def home @home end |
#install_extension_in_lib ⇒ Object
Install extensions into lib as well as into the extension directory.
161 162 163 |
# File 'lib/rubygems/config_file.rb', line 161 def install_extension_in_lib @install_extension_in_lib end |
#ipv4_fallback_enabled ⇒ Object
Experimental ==
Fallback to IPv4 when IPv6 is not reachable or slow (default: false)
167 168 169 |
# File 'lib/rubygems/config_file.rb', line 167 def ipv4_fallback_enabled @ipv4_fallback_enabled end |
#path ⇒ Object
Where to look for gems (deprecated)
98 99 100 |
# File 'lib/rubygems/config_file.rb', line 98 def path @path end |
#sources ⇒ Object
sources to look for gems
151 152 153 |
# File 'lib/rubygems/config_file.rb', line 151 def sources @sources end |
#ssl_ca_cert ⇒ Object
Path name of directory or file of openssl CA certificate, used for remote https connection
147 148 149 |
# File 'lib/rubygems/config_file.rb', line 147 def ssl_ca_cert @ssl_ca_cert end |
#ssl_client_cert ⇒ Object (readonly)
Path name of directory or file of openssl client certificate, used for remote https connection with client authentication
172 173 174 |
# File 'lib/rubygems/config_file.rb', line 172 def ssl_client_cert @ssl_client_cert end |
#ssl_verify_mode ⇒ Object (readonly)
openssl verify mode value, used for remote https connection
141 142 143 |
# File 'lib/rubygems/config_file.rb', line 141 def ssl_verify_mode @ssl_verify_mode end |
#update_sources ⇒ Object
True if we want to update the SourceInfoCache every time, false otherwise
132 133 134 |
# File 'lib/rubygems/config_file.rb', line 132 def update_sources @update_sources end |
#verbose ⇒ Object
Verbose level of output:
- false -- No output
- true -- Normal output
- :loud -- Extra output
122 123 124 |
# File 'lib/rubygems/config_file.rb', line 122 def verbose @verbose end |
Class Method Details
.deep_transform_config_keys!(config) ⇒ Object
574 575 576 577 578 579 580 581 582 583 584 585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 608 609 610 611 612 |
# File 'lib/rubygems/config_file.rb', line 574 def self.deep_transform_config_keys!(config) config.transform_keys! do |k| if k.match?(/\A:(.*)\z/) k[1..-1].to_sym elsif k.include?("__") || k.end_with?("/") if k.is_a?(Symbol) k.to_s.gsub(/__/,".").delete_suffix("/").to_sym else k.dup.gsub(/__/,".").delete_suffix("/") end else k end end config.transform_values! do |v| if v.is_a?(String) if v.match?(/\A:(.*)\z/) v[1..-1].to_sym elsif v.match?(/\A[+-]?\d+\z/) v.to_i elsif v.match?(/\A(?:true|false)\z/) v == "true" elsif v.empty? nil else v end elsif v.empty? nil elsif v.is_a?(Hash) deep_transform_config_keys!(v) else v end end config end |
.dump_with_rubygems_yaml(content) ⇒ Object
556 557 558 559 560 561 562 563 |
# File 'lib/rubygems/config_file.rb', line 556 def self.dump_with_rubygems_yaml(content) content.transform_keys! do |k| k.is_a?(Symbol) ? ":#{k}" : k end require_relative "yaml_serializer" Gem::YAMLSerializer.dump(content) end |
.load_with_rubygems_config_hash(yaml) ⇒ Object
565 566 567 568 569 570 |
# File 'lib/rubygems/config_file.rb', line 565 def self.load_with_rubygems_config_hash(yaml) require_relative "yaml_serializer" content = Gem::YAMLSerializer.load(yaml) deep_transform_config_keys!(content) end |
Instance Method Details
#==(other) ⇒ Object
:nodoc:
544 545 546 547 548 549 550 551 |
# File 'lib/rubygems/config_file.rb', line 544 def ==(other) # :nodoc: self.class === other && @backtrace == other.backtrace && @bulk_threshold == other.bulk_threshold && @verbose == other.verbose && @update_sources == other.update_sources && @hash == other.hash end |
#[](key) ⇒ Object
Return the configuration information for key.
535 536 537 |
# File 'lib/rubygems/config_file.rb', line 535 def [](key) @hash[key] || @hash[key.to_s] end |
#[]=(key, value) ⇒ Object
Set configuration option key to value.
540 541 542 |
# File 'lib/rubygems/config_file.rb', line 540 def []=(key, value) @hash[key] = value end |
#api_keys ⇒ Object
Hash of RubyGems.org and alternate API keys
262 263 264 265 266 |
# File 'lib/rubygems/config_file.rb', line 262 def api_keys load_api_keys unless @api_keys @api_keys end |
#check_credentials_permissions ⇒ Object
Checks the permissions of the credentials file. If they are not 0600 an error message is displayed and RubyGems aborts.
272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 |
# File 'lib/rubygems/config_file.rb', line 272 def return if Gem.win_platform? # windows doesn't write 0600 as 0600 return unless File.exist? credentials_path = File.stat(credentials_path).mode & 0o777 return if == 0o600 alert_error <<-ERROR Your gem push credentials file located at: \t#{credentials_path} has file permissions of 0#{.to_s 8} but 0600 is required. To fix this error run: \tchmod 0600 #{credentials_path} You should reset your credentials at: \thttps://rubygems.org/profile/edit if you believe they were disclosed to a third party. ERROR terminate_interaction 1 end |
#config_file_name ⇒ Object
The name of the configuration file.
418 419 420 |
# File 'lib/rubygems/config_file.rb', line 418 def config_file_name @config_file_name || Gem.config_file end |
#credentials_path ⇒ Object
Location of RubyGems.org credentials
304 305 306 307 308 309 310 311 |
# File 'lib/rubygems/config_file.rb', line 304 def credentials_path credentials = File.join Gem.user_home, ".gem", "credentials" if File.exist? credentials credentials else File.join Gem.data_home, "gem", "credentials" end end |
#each {|:update_sources, @update_sources| ... } ⇒ Object
Delegates to @hash
442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 457 |
# File 'lib/rubygems/config_file.rb', line 442 def each(&block) hash = @hash.dup hash.delete :update_sources hash.delete :verbose hash.delete :backtrace hash.delete :bulk_threshold yield :update_sources, @update_sources yield :verbose, @verbose yield :backtrace, @backtrace yield :bulk_threshold, @bulk_threshold yield "config_file_name", @config_file_name if @config_file_name hash.each(&block) end |
#handle_arguments(arg_list) ⇒ Object
Handle the command arguments.
460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 |
# File 'lib/rubygems/config_file.rb', line 460 def handle_arguments(arg_list) @args = [] arg_list.each do |arg| case arg when /^--(backtrace|traceback)$/ then @backtrace = true when /^--debug$/ then $DEBUG = true warn "NOTE: Debugging mode prints all exceptions even when rescued" else @args << arg end end end |
#last_update_check ⇒ Object
Reads time of last update check from state file
428 429 430 431 432 433 434 |
# File 'lib/rubygems/config_file.rb', line 428 def last_update_check if File.readable?(state_file_name) File.read(state_file_name).to_i else 0 end end |
#last_update_check=(timestamp) ⇒ Object
Writes time of last update check to state file
437 438 439 |
# File 'lib/rubygems/config_file.rb', line 437 def last_update_check=() File.write(state_file_name, .to_s) if state_file_writable? end |
#load_api_keys ⇒ Object
313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 |
# File 'lib/rubygems/config_file.rb', line 313 def load_api_keys @api_keys = if File.exist? credentials_path load_file(credentials_path) else @hash end if @api_keys.key? :rubygems_api_key @rubygems_api_key = @api_keys[:rubygems_api_key] @api_keys[:rubygems] = @api_keys.delete :rubygems_api_key unless @api_keys.key? :rubygems end end |
#load_file(filename) ⇒ Object
376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 |
# File 'lib/rubygems/config_file.rb', line 376 def load_file(filename) yaml_errors = [ArgumentError] return {} unless filename && !filename.empty? && File.exist?(filename) begin config = self.class.load_with_rubygems_config_hash(File.read(filename)) if config.keys.any? {|k| k.to_s.gsub(%r{https?:\/\/}, "").include?(": ") } warn "Failed to load #{filename} because it doesn't contain valid YAML hash" return {} else return config end rescue *yaml_errors => e warn "Failed to load #{filename}, #{e}" rescue Errno::EACCES warn "Failed to load #{filename} due to permissions problem." end {} end |
#really_verbose ⇒ Object
Really verbose mode gives you extra output.
478 479 480 481 482 483 484 485 |
# File 'lib/rubygems/config_file.rb', line 478 def really_verbose case verbose when true, false, nil then false else true end end |
#rubygems_api_key ⇒ Object
Returns the RubyGems.org API key
332 333 334 335 336 |
# File 'lib/rubygems/config_file.rb', line 332 def rubygems_api_key load_api_keys unless @rubygems_api_key @rubygems_api_key end |
#rubygems_api_key=(api_key) ⇒ Object
Sets the RubyGems.org API key to api_key
341 342 343 344 345 |
# File 'lib/rubygems/config_file.rb', line 341 def rubygems_api_key=(api_key) set_api_key :rubygems_api_key, api_key @rubygems_api_key = api_key end |
#set_api_key(host, api_key) ⇒ Object
Set a specific host's API key to api_key
350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 |
# File 'lib/rubygems/config_file.rb', line 350 def set_api_key(host, api_key) config = load_file(credentials_path).merge(host => api_key) dirname = File.dirname credentials_path require "fileutils" FileUtils.mkdir_p(dirname) = 0o600 & ~File.umask File.open(credentials_path, "w", ) do |f| f.write self.class.dump_with_rubygems_yaml(config) end load_api_keys # reload end |
#state_file_name ⇒ Object
The name of the state file.
423 424 425 |
# File 'lib/rubygems/config_file.rb', line 423 def state_file_name Gem.state_file end |
#state_file_writable? ⇒ Boolean
Check state file is writable. Creates empty file if not present to ensure we can write to it.
404 405 406 407 408 409 410 411 412 413 414 415 |
# File 'lib/rubygems/config_file.rb', line 404 def state_file_writable? if File.exist?(state_file_name) File.writable?(state_file_name) else require "fileutils" FileUtils.mkdir_p File.dirname(state_file_name) File.open(state_file_name, "w") {} true end rescue Errno::EACCES false end |
#to_yaml ⇒ Object
to_yaml only overwrites things you can't override on the command line.
488 489 490 491 492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 |
# File 'lib/rubygems/config_file.rb', line 488 def to_yaml # :nodoc: yaml_hash = {} yaml_hash[:backtrace] = @hash.fetch(:backtrace, DEFAULT_BACKTRACE) yaml_hash[:bulk_threshold] = @hash.fetch(:bulk_threshold, DEFAULT_BULK_THRESHOLD) yaml_hash[:sources] = Gem.sources.to_a yaml_hash[:update_sources] = @hash.fetch(:update_sources, DEFAULT_UPDATE_SOURCES) yaml_hash[:verbose] = @hash.fetch(:verbose, DEFAULT_VERBOSITY) yaml_hash[:concurrent_downloads] = @hash.fetch(:concurrent_downloads, DEFAULT_CONCURRENT_DOWNLOADS) yaml_hash[:install_extension_in_lib] = @hash.fetch(:install_extension_in_lib, DEFAULT_INSTALL_EXTENSION_IN_LIB) yaml_hash[:ssl_verify_mode] = @hash[:ssl_verify_mode] if @hash.key? :ssl_verify_mode yaml_hash[:ssl_ca_cert] = @hash[:ssl_ca_cert] if @hash.key? :ssl_ca_cert yaml_hash[:ssl_client_cert] = @hash[:ssl_client_cert] if @hash.key? :ssl_client_cert keys = yaml_hash.keys.map(&:to_s) keys << "debug" re = Regexp.union(*keys) @hash.each do |key, value| key = key.to_s next if key&.match?(re) yaml_hash[key.to_s] = value end self.class.dump_with_rubygems_yaml(yaml_hash) end |
#unset_api_key! ⇒ Object
Remove the ~/.gem/credentials file to clear all the current sessions.
370 371 372 373 374 |
# File 'lib/rubygems/config_file.rb', line 370 def unset_api_key! return false unless File.exist?(credentials_path) File.delete(credentials_path) end |
#write ⇒ Object
Writes out this config file, replacing its source.
525 526 527 528 529 530 531 532 |
# File 'lib/rubygems/config_file.rb', line 525 def write require "fileutils" FileUtils.mkdir_p File.dirname(config_file_name) File.open config_file_name, "w" do |io| io.write to_yaml end end |