Class: Gem::Package::TarReader
- Inherits:
-
Object
- Object
- Gem::Package::TarReader
- Includes:
- Enumerable
- Defined in:
- lib/rubygems/package/tar_reader.rb
Overview
TarReader reads tar files and allows iteration over their items
Defined Under Namespace
Classes: Entry
Instance Attribute Summary collapse
-
#io ⇒ Object
readonly
:nodoc:.
Class Method Summary collapse
-
.new(io) ⇒ Object
Creates a new TarReader on
ioand yields it to the block, if given.
Instance Method Summary collapse
-
#close ⇒ Object
Close the tar file.
-
#each ⇒ Object
(also: #each_entry)
Iterates over files in the tarball yielding each entry.
-
#initialize(io) ⇒ TarReader
constructor
Creates a new tar file reader on
iowhich needs to respond to #pos, #eof?, #read, #getc and #pos=. -
#rewind ⇒ Object
NOTE: Do not call #rewind during #each.
-
#seek(name) ⇒ Object
Seeks through the tar file until it finds the
entrywithnameand yields it.
Constructor Details
#initialize(io) ⇒ TarReader
Creates a new tar file reader on io which needs to respond to #pos, #eof?, #read, #getc and #pos=
39 40 41 42 |
# File 'lib/rubygems/package/tar_reader.rb', line 39 def initialize(io) @io = io @init_pos = io.pos end |
Instance Attribute Details
#io ⇒ Object (readonly)
:nodoc:
33 34 35 |
# File 'lib/rubygems/package/tar_reader.rb', line 33 def io @io end |
Class Method Details
.new(io) ⇒ Object
Creates a new TarReader on io and yields it to the block, if given.
19 20 21 22 23 24 25 26 27 28 29 30 31 |
# File 'lib/rubygems/package/tar_reader.rb', line 19 def self.new(io) reader = super return reader unless block_given? begin yield reader ensure reader.close end nil end |
Instance Method Details
#close ⇒ Object
Close the tar file
47 48 |
# File 'lib/rubygems/package/tar_reader.rb', line 47 def close end |
#each ⇒ Object Also known as: each_entry
Iterates over files in the tarball yielding each entry
53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 |
# File 'lib/rubygems/package/tar_reader.rb', line 53 def each return enum_for __method__ unless block_given? until @io.eof? do begin header = Gem::Package::TarHeader.from @io rescue ArgumentError => e # Specialize only exceptions from Gem::Package::TarHeader.strict_oct raise e unless e..match?(/ is not an octal string$/) raise Gem::Package::TarInvalidError, e. end return if header.empty? entry = Gem::Package::TarReader::Entry.new header, @io yield entry entry.close end end |
#rewind ⇒ Object
NOTE: Do not call #rewind during #each
77 78 79 80 81 82 83 |
# File 'lib/rubygems/package/tar_reader.rb', line 77 def rewind if @init_pos == 0 @io.rewind else @io.pos = @init_pos end end |
#seek(name) ⇒ Object
Seeks through the tar file until it finds the entry with name and yields it. Rewinds the tar file to the beginning when the block terminates.
90 91 92 93 94 95 96 97 98 99 100 |
# File 'lib/rubygems/package/tar_reader.rb', line 90 def seek(name) # :yields: entry found = find do |entry| entry.full_name == name end return unless found yield found ensure rewind end |