48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
|
# File 'lib/rubygems/commands/unpack_command.rb', line 48
def description
"The unpack command allows you to examine the contents of a gem or modify\nthem to help diagnose a bug.\n\nYou can add the contents of the unpacked gem to the load path using the\nRUBYLIB environment variable or -I:\n\n$ gem unpack my_gem\nUnpacked gem: '.../my_gem-1.0'\n[edit my_gem-1.0/lib/my_gem.rb]\n$ ruby -Imy_gem-1.0/lib -S other_program\n\nYou can repackage an unpacked gem using the build command. See the build\ncommand help for an example.\n EOF\nend\n"
|