Class: Gem::Ext::Builder
- Inherits:
-
Object
- Object
- Gem::Ext::Builder
- Includes:
- UserInteraction
- Defined in:
- lib/rubygems/ext/builder.rb
Direct Known Subclasses
CargoBuilder, CmakeBuilder, ConfigureBuilder, ExtConfBuilder, RakeBuilder
Defined Under Namespace
Classes: NoMakefileError
Instance Attribute Summary collapse
-
#build_args ⇒ Object
:nodoc:.
Class Method Summary collapse
- .class_name ⇒ Object
- .make(dest_path, results, make_dir = Dir.pwd, sitedir = nil, targets = ["clean", "", "install"], target_rbconfig: Gem.target_rbconfig, n_jobs: nil) ⇒ Object
- .ruby ⇒ Object
- .run(command, results, command_name = nil, dir = Dir.pwd, env = {}) {|status, results| ... } ⇒ Object
- .shelljoin(command) ⇒ Object
- .shellsplit(command) ⇒ Object
-
.shellsplit_command(command) ⇒ Object
Splits a command string such as ENV into an argument list.
Instance Method Summary collapse
-
#build_error(output, backtrace = nil) ⇒ Object
Logs the build
output, then raises Gem::Ext::BuildError. -
#build_extension(extension, dest_path) ⇒ Object
:nodoc:.
-
#build_extensions ⇒ Object
Builds extensions.
-
#builder_for(extension) ⇒ Object
Chooses the extension builder class for
extension. -
#initialize(spec, build_args = spec.build_args, target_rbconfig = Gem.target_rbconfig, build_jobs = nil) ⇒ Builder
constructor
Creates a new extension builder for
spec. -
#write_gem_make_out(output) ⇒ Object
Writes
outputto gem_make.out in the extension install directory.
Methods included from UserInteraction
#alert, #alert_error, #alert_warning, #ask, #ask_for_password, #ask_yes_no, #choose_from_list, #say, #terminate_interaction, #verbose
Methods included from DefaultUserInteraction
ui, #ui, ui=, #ui=, use_ui, #use_ui
Methods included from Text
#clean_text, #format_text, #levenshtein_distance, #min3, #truncate_text
Constructor Details
#initialize(spec, build_args = spec.build_args, target_rbconfig = Gem.target_rbconfig, build_jobs = nil) ⇒ Builder
Creates a new extension builder for spec. If the spec does not yet
have build arguments, saved, set build_args which is an ARGV-style
array.
181 182 183 184 185 186 187 |
# File 'lib/rubygems/ext/builder.rb', line 181 def initialize(spec, build_args = spec.build_args, target_rbconfig = Gem.target_rbconfig, build_jobs = nil) @spec = spec @build_args = build_args @gem_dir = spec.full_gem_path @target_rbconfig = target_rbconfig @build_jobs = build_jobs end |
Instance Attribute Details
#build_args ⇒ Object
:nodoc:
17 18 19 |
# File 'lib/rubygems/ext/builder.rb', line 17 def build_args @build_args end |
Class Method Details
.class_name ⇒ Object
19 20 21 22 |
# File 'lib/rubygems/ext/builder.rb', line 19 def self.class_name name =~ /Ext::(.*)Builder/ $1.downcase end |
.make(dest_path, results, make_dir = Dir.pwd, sitedir = nil, targets = ["clean", "", "install"], target_rbconfig: Gem.target_rbconfig, n_jobs: nil) ⇒ Object
24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 |
# File 'lib/rubygems/ext/builder.rb', line 24 def self.make(dest_path, results, make_dir = Dir.pwd, sitedir = nil, targets = ["clean", "", "install"], target_rbconfig: Gem.target_rbconfig, n_jobs: nil) unless File.exist? File.join(make_dir, "Makefile") # No makefile exists, nothing to do. raise NoMakefileError, "No Makefile found in #{make_dir}" end # try to find make program from Ruby configure arguments first target_rbconfig["configure_args"] =~ /with-make-prog\=(\w+)/ make_program_name = ENV["MAKE"] || ENV["make"] || $1 make_program_name ||= RUBY_PLATFORM.include?("mswin") ? "nmake" : "make" make_program = shellsplit_command(make_program_name) is_nmake = /\bnmake/i.match?(make_program_name) # The installation of the bundled gems is failed when DESTDIR is empty in mswin platform. destdir = !is_nmake || ENV["DESTDIR"] && ENV["DESTDIR"] != "" ? format("DESTDIR=%s", ENV["DESTDIR"]) : "" # nmake doesn't support parallel build unless is_nmake have_make_arguments = make_program.size > 1 n_jobs ||= 0 if !have_make_arguments && n_jobs > 1 && !ENV["MAKEFLAGS"]&.match(/-j\d*(\s|\Z)/) make_program << "-j#{n_jobs}" end end env = [destdir] if sitedir env << format("sitearchdir=%s", sitedir) env << format("sitelibdir=%s", sitedir) end targets.each do |target| # Pass DESTDIR via command line to override what's in MAKEFLAGS cmd = [ *make_program, *env, target, ].reject(&:empty?) begin run(cmd, results, "make #{target}".rstrip, make_dir) rescue Gem::InstallError raise unless target == "clean" # ignore clean failure end end end |
.ruby ⇒ Object
73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 |
# File 'lib/rubygems/ext/builder.rb', line 73 def self.ruby # Gem.ruby is quoted if it contains whitespace cmd = shellsplit(Gem.ruby) # This load_path is only needed when running rubygems test without a proper installation. # Prepending it in a normal installation will cause problem with order of $LOAD_PATH. # Therefore only add load_path if it is not present in the default $LOAD_PATH. load_path = File.("../..", __dir__) case load_path when RbConfig::CONFIG["sitelibdir"], RbConfig::CONFIG["vendorlibdir"], RbConfig::CONFIG["rubylibdir"] cmd else cmd << "-I#{load_path}" end end |
.run(command, results, command_name = nil, dir = Dir.pwd, env = {}) {|status, results| ... } ⇒ Object
89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 |
# File 'lib/rubygems/ext/builder.rb', line 89 def self.run(command, results, command_name = nil, dir = Dir.pwd, env = {}) verbose = Gem.configuration.really_verbose begin rubygems_gemdeps = ENV["RUBYGEMS_GEMDEPS"] ENV["RUBYGEMS_GEMDEPS"] = nil if verbose puts("current directory: #{dir}") p(command) end results << "current directory: #{dir}" results << shelljoin(command) require "open3" # Set $SOURCE_DATE_EPOCH for the subprocess. build_env = { "SOURCE_DATE_EPOCH" => Gem.source_date_epoch_string }.merge(env) # A single-element command would be parsed as a shell command line, # splitting an unquoted command path containing spaces. Use the # [cmdname, argv0] form to keep exec semantics. command = [[command.first, command.first]] if command.size == 1 output, status = begin Open3.popen2e(build_env, *command, chdir: dir) do |stdin, stdouterr, wait_thread| stdin.close output = String.new while line = stdouterr.gets output << line if verbose print line end end [output, wait_thread.value] end rescue StandardError => error raise Gem::InstallError, "#{command_name || class_name} failed#{error.}" end unless verbose results << output end ensure ENV["RUBYGEMS_GEMDEPS"] = rubygems_gemdeps end unless status.success? results << "Building has failed. See above output for more information on the failure." if verbose end yield(status, results) if block_given? unless status.success? exit_reason = if status.exited? ", exit code #{status.exitstatus}" elsif status.signaled? ", uncaught signal #{status.termsig}" end raise Gem::InstallError, "#{command_name || class_name} failed#{exit_reason}" end end |
.shelljoin(command) ⇒ Object
170 171 172 173 174 |
# File 'lib/rubygems/ext/builder.rb', line 170 def self.shelljoin(command) require "shellwords" Shellwords.join(command) end |
.shellsplit(command) ⇒ Object
149 150 151 152 153 |
# File 'lib/rubygems/ext/builder.rb', line 149 def self.shellsplit(command) require "shellwords" Shellwords.split(command) end |
.shellsplit_command(command) ⇒ Object
Splits a command string such as ENV into an argument list.
On Windows, a value that names an existing file, such as C:\path\nmake.exe, is taken as a single word because POSIX shell splitting would consume the backslashes as escape characters. Any other value is split like a POSIX shell command line, where a path containing spaces must be quoted.
164 165 166 167 168 |
# File 'lib/rubygems/ext/builder.rb', line 164 def self.shellsplit_command(command) return [command] if Gem.win_platform? && File.file?(command) shellsplit(command) end |
Instance Method Details
#build_error(output, backtrace = nil) ⇒ Object
Logs the build output, then raises Gem::Ext::BuildError.
212 213 214 215 216 217 218 219 220 221 222 223 224 225 |
# File 'lib/rubygems/ext/builder.rb', line 212 def build_error(output, backtrace = nil) # :nodoc: gem_make_out = write_gem_make_out output = <<-EOF ERROR: Failed to build gem native extension. #{output} Gem files will remain installed in #{@gem_dir} for inspection. Results logged to #{gem_make_out} EOF raise Gem::Ext::BuildError, , backtrace end |
#build_extension(extension, dest_path) ⇒ Object
:nodoc:
227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 |
# File 'lib/rubygems/ext/builder.rb', line 227 def build_extension(extension, dest_path) # :nodoc: results = [] builder = builder_for(extension) extension_dir = File. File.join(@gem_dir, File.dirname(extension)) lib_dir = File.join @spec.full_gem_path, @spec.raw_require_paths.first begin FileUtils.mkdir_p dest_path results = builder.build(extension, dest_path, results, @build_args, lib_dir, extension_dir, @target_rbconfig, n_jobs: @build_jobs) verbose { results.join("\n") } write_gem_make_out results.join "\n" rescue StandardError => e results << e. build_error(results.join("\n"), $@) end end |
#build_extensions ⇒ Object
Builds extensions. Valid types of extensions are extconf.rb files, configure scripts and rakefiles or mkrf_conf files.
255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 |
# File 'lib/rubygems/ext/builder.rb', line 255 def build_extensions return if @spec.extensions.empty? if @build_args.empty? say "Building native extensions. This could take a while..." else say "Building native extensions with: '#{@build_args.join " "}'" say "This could take a while..." end dest_path = @spec.extension_dir require "fileutils" FileUtils.rm_f @spec.gem_build_complete_path @spec.extensions.each do |extension| build_extension extension, dest_path end FileUtils.touch @spec.gem_build_complete_path end |
#builder_for(extension) ⇒ Object
Chooses the extension builder class for extension
192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 |
# File 'lib/rubygems/ext/builder.rb', line 192 def builder_for(extension) # :nodoc: case extension when /extconf/ then Gem::Ext::ExtConfBuilder when /configure/ then Gem::Ext::ConfigureBuilder when /rakefile/i, /mkrf_conf/i then Gem::Ext::RakeBuilder when /CMakeLists.txt/ then Gem::Ext::CmakeBuilder.new when /Cargo.toml/ then Gem::Ext::CargoBuilder.new else build_error("No builder for extension '#{extension}'") end end |
#write_gem_make_out(output) ⇒ Object
Writes output to gem_make.out in the extension install directory.
280 281 282 283 284 285 286 287 288 289 290 |
# File 'lib/rubygems/ext/builder.rb', line 280 def write_gem_make_out(output) # :nodoc: destination = File.join @spec.extension_dir, "gem_make.out" FileUtils.mkdir_p @spec.extension_dir File.open destination, "wb" do |io| io.puts output end destination end |